#include <iostream>
#include <curl/curl.h>

// 此代码依赖开源库libcurl, 请自行下载源码编译并引用，源码地址: https://curl.haxx.se/download.html

const std::string ProxyHost = "flow.hailiangip.com";
const int HttpProxyPort = 14223;
const int Socks5ProxyPort = 14224;
const std::string DestUrl = "http://www.baidu.com";
const std::string OrderId = ""; // 订单号
const std::string Pid = "-1"; // 省份id，-1表示随机
const std::string Cid = "-1"; // 城市id，-1表示随机
const std::string Sip = "0"; // 是否切换ip，0表示自动切换，1表示不能切换，默认0
const std::string Uid = ""; // 自定义，dik情况下，相同的uid会尽可能采用相同的ip，可以认为是同一组会话，不填表示每次请求都随机
const std::string ZoneId = ""; // 通道ID
const std::string PWD = ""; // 订单密码

size_t WriteCallback(void* contents, size_t size, size_t nmemb, std::string* response) {
    size_t totalSize = size * nmemb;
    response->append((char*)contents, totalSize);
    return totalSize;
}

void httpProxyWithPassRequest(const std::string& destUrl, const std::string& username, const std::string& proxyPassword) {
    std::string proxyUrl = "http://" + username + ":" + proxyPassword + "@" + ProxyHost + ":" + std::to_string(HttpProxyPort);
    
    CURL* curl = curl_easy_init();
    if (curl) {
        curl_easy_setopt(curl, CURLOPT_PROXY, proxyUrl.c_str());
        curl_easy_setopt(curl, CURLOPT_URL, destUrl.c_str());
        
        std::string response;
        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, WriteCallback);
        curl_easy_setopt(curl, CURLOPT_WRITEDATA, &response);
        
        CURLcode res = curl_easy_perform(curl);
        if (res != CURLE_OK) {
            std::cerr << "Failed to perform HTTP request: " << curl_easy_strerror(res) << std::endl;
        } else {
            long httpCode = 0;
            curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &httpCode);
            std::cout << "HTTP代理请求状态码: " << httpCode << std::endl;
            std::cout << "HTTP代理请求响应内容: " << response << std::endl;
        }
        
        curl_easy_cleanup(curl);
    }
}

void socks5ProxyWithPassRequest(const std::string& destUrl, const std::string& username, const std::string& proxyPassword) {
    std::string proxyUrl = "socks5://" + username + ":" + proxyPassword + "@" + ProxyHost + ":" + std::to_string(Socks5ProxyPort);

    CURL* curl = curl_easy_init();
    if (curl) {
        curl_easy_setopt(curl, CURLOPT_PROXY, proxyUrl.c_str());
        curl_easy_setopt(curl, CURLOPT_PROXYTYPE, CURLPROXY_SOCKS5); // 设置代理类型为SOCKS5

        curl_easy_setopt(curl, CURLOPT_URL, destUrl.c_str());

        std::string response;
        curl_easy_setopt(curl, CURLOPT_WRITEFUNCTION, WriteCallback);
        curl_easy_setopt(curl, CURLOPT_WRITEDATA, &response);

        CURLcode res = curl_easy_perform(curl);
        if (res != CURLE_OK) {
            std::cerr << "Failed to perform socks5 request: " << curl_easy_strerror(res) << std::endl;
        } else {
            long httpCode = 0;
            curl_easy_getinfo(curl, CURLINFO_RESPONSE_CODE, &httpCode);
            std::cout << "socks5代理请求状态码: " << httpCode << std::endl;
            std::cout << "socks5代理请求响应内容: " << response << std::endl;
        }
        
        curl_easy_cleanup(curl);
    }
}

int main() {
    std::string username = OrderId;
    std::string password;

    // 默认账密模式
    password += "pwd=" + PWD + "&pid=" + Pid + "&cid=" + Cid + "&uid=" + Uid + "&sip=" + Sip;
    
    httpProxyWithPassRequest(DestUrl, username, password);
    socks5ProxyWithPassRequest(DestUrl, username, password);
    
    return 0;
}